@@ -1,8 +1,6 @@
-context("binds")
-
 # columns -----------------------------------------------------------------
 
-test_that("cbind uses shallow copies", {
+test_that("bind_cols() uses shallow copies", {
   df1 <- data.frame(
     int = 1:10,
     num = rnorm(10),
@@ -20,7 +18,7 @@ test_that("cbind uses shallow copies", {
   expect_equal(lobstr::obj_addrs(df2), lobstr::obj_addrs(df[names(df2)]))
 })
 
-test_that("bind_cols handles lists (#1104)", {
+test_that("bind_cols() handles lists (#1104)", {
   exp <- tibble(x = 1, y = "a", z = 2)
 
   l1 <- list(x = 1, y = "a")
@@ -30,28 +28,30 @@ test_that("bind_cols handles lists (#1104)", {
   expect_identical(bind_cols(list(l1, l2)), exp)
 })
 
-test_that("bind_cols handles empty argument list (#1963)", {
+test_that("bind_cols() handles empty argument list (#1963)", {
   expect_equal(bind_cols(), tibble())
 })
 
-test_that("bind_cols handles all-NULL values (#2303)", {
+test_that("bind_cols() handles all-NULL values (#2303)", {
   expect_identical(bind_cols(list(a = NULL, b = NULL)), tibble())
   expect_identical(bind_cols(NULL), tibble())
 })
 
-test_that("bind_cols repairs names", {
+test_that("bind_cols() repairs names", {
   df <- tibble(a = 1, b = 2)
-  bound <- bind_cols(df, df)
+  expect_snapshot(bound <- bind_cols(df, df))
 
-  repaired <- as_tibble(
-    data.frame(a = 1, b = 2, a = 1, b = 2, check.names = FALSE),
-    .name_repair = "unique"
+  expect_message(
+    repaired <- as_tibble(
+      data.frame(a = 1, b = 2, a = 1, b = 2, check.names = FALSE),
+      .name_repair = "unique"
+    ), "New names"
   )
 
   expect_identical(bound, repaired)
 })
 
-test_that("bind_cols unpacks tibbles", {
+test_that("bind_cols() unpacks tibbles", {
   expect_equal(
     bind_cols(list(y = tibble(x = 1:2))),
     tibble(x = 1:2)
@@ -63,7 +63,7 @@ test_that("bind_cols unpacks tibbles", {
 })
 
 test_that("bind_cols() honours .name_repair=", {
-  res <- expect_message(bind_cols(
+  expect_message(res <- bind_cols(
     data.frame(a = 1), data.frame(a = 2)
   ))
   expect_equal(res, data.frame(a...1 = 1, a...2 = 2))
@@ -94,7 +94,7 @@ test_that("bind_rows() equivalent to rbind()", {
   expect_equal(res, exp)
 })
 
-test_that("bind_rows reorders columns", {
+test_that("bind_rows() reorders columns", {
   df_var_scramble <- df_var[sample(ncol(df_var))]
 
   expect_equal(
@@ -103,27 +103,27 @@ test_that("bind_rows reorders columns", {
   )
 })
 
-test_that("bind_rows ignores NULL", {
+test_that("bind_rows() ignores NULL", {
   df <- tibble(a = 1)
 
   expect_equal(bind_rows(df, NULL), df)
   expect_equal(bind_rows(list(df, NULL)), df)
 })
 
-test_that("bind_rows handles list columns (#463)", {
+test_that("bind_rows() handles list columns (#463)", {
   dfl <- tibble(x = list(1:2, 1:3, 1:4))
   res <- bind_rows(list(dfl, dfl))
   expect_equal(rep(dfl$x, 2L), res$x)
 })
 
-test_that("can bind lists of data frames #1389", {
+test_that("bind_rows() handles lists of data frames #1389", {
   df <- tibble(x = 1)
 
   res <- bind_rows(list(df, df), list(df, df))
   expect_equal(nrow(res), 4)
 })
 
-test_that("bind_rows handles data frames with no rows (#597)", {
+test_that("bind_rows() handles data frames with no rows (#597)", {
   df1 <- tibble(x = 1, y = factor("a"))
   df0 <- df1[0, ]
 
@@ -132,7 +132,7 @@ test_that("bind_rows handles data frames with no rows (#597)", {
   expect_identical(bind_rows(df0, df1), df1)
 })
 
-test_that("bind_rows handles data frames with no columns (#1346)", {
+test_that("bind_rows() handles data frames with no columns (#1346)", {
   df1 <- tibble(x = 1, y = factor("a"))
   df0 <- df1[, 0]
 
@@ -143,7 +143,7 @@ test_that("bind_rows handles data frames with no columns (#1346)", {
   expect_equal(res$x, c(NA, 1))
 })
 
-test_that("bind_rows handles lists with NULL values (#2056)", {
+test_that("bind_rows() handles lists with NULL values (#2056)", {
   df1 <- tibble(x = 1, y = 1)
   df2 <- tibble(x = 2, y = 2)
   lst1 <- list(a = df1, NULL, b = df2)
@@ -157,11 +157,11 @@ test_that("bind_rows handles lists with NULL values (#2056)", {
   expect_identical(bind_rows(lst1, .id = "names"), df3)
 })
 
-test_that("bind_rows handles lists with list() values (#2826)", {
+test_that("bind_rows() handles lists with list() values (#2826)", {
   expect_equal(bind_rows(list(iris, list())), iris)
 })
 
-test_that("bind_rows puts data frames in order received even if no columns (#2175)", {
+test_that("bind_rows() puts data frames in order received even if no columns (#2175)", {
   df2 <- tibble(x = 2, y = "b")
   df1 <- df2[, 0]
 
@@ -180,7 +180,7 @@ test_that("bind_rows(.id= NULL) does not set names (#5089)", {
 
 # Column coercion --------------------------------------------------------------
 
-test_that("bind_rows promotes integer to numeric", {
+test_that("bind_rows() promotes integer to numeric", {
   df1 <- tibble(a = 1L, b = 1L)
   df2 <- tibble(a = 1, b = 1L)
 
@@ -189,7 +189,7 @@ test_that("bind_rows promotes integer to numeric", {
   expect_equal(typeof(res$b), "integer")
 })
 
-test_that("bind_rows promotes factor to character with warning", {
+test_that("bind_rows() promotes factor to character with warning", {
   df1 <- tibble(a = factor("a"))
   df2 <- tibble(a = "b")
 
@@ -197,7 +197,7 @@ test_that("bind_rows promotes factor to character with warning", {
   expect_equal(typeof(res$a), "character")
 })
 
-test_that("bind_rows coerces factor when levels don't match", {
+test_that("bind_rows() coerces factor when levels don't match", {
   df1 <- data.frame(a = factor("a"))
   df2 <- data.frame(a = factor("b"))
 
@@ -205,7 +205,7 @@ test_that("bind_rows coerces factor when levels don't match", {
   expect_equal(res$a, factor(c("a", "b")))
 })
 
-test_that("bind_rows handles NA in factors #279", {
+test_that("bind_rows() handles NA in factors #279", {
   df1 <- tibble(a = factor("a"))
   df2 <- tibble(a = factor(NA))
 
@@ -213,7 +213,7 @@ test_that("bind_rows handles NA in factors #279", {
   expect_equal(res$a, factor(c("a", NA)))
 })
 
-test_that("bind_rows preserves timezones #298", {
+test_that("bind_rows() preserves timezones #298", {
   dates1 <- data.frame(
     ID = c("a", "b", "c"),
     dates = structure(c(-247320000, -246196800, -245073600),
@@ -236,14 +236,14 @@ test_that("bind_rows preserves timezones #298", {
   expect_equal(attr(alldates$dates, "tzone"), "GMT")
 })
 
-test_that("bind_rows handles all NA columns (#493)", {
+test_that("bind_rows() handles all NA columns (#493)", {
   mydata <- list(
     data.frame(x = c("foo", "bar"), stringsAsFactors = TRUE),
     data.frame(x = NA)
   )
   res <- bind_rows(mydata)
   expect_true(is.na(res$x[3]))
-  expect_is(res$x, "factor")
+  expect_s3_class(res$x, "factor")
 
   mydata <- list(
     data.frame(x = NA),
@@ -251,10 +251,10 @@ test_that("bind_rows handles all NA columns (#493)", {
   )
   res <- bind_rows(mydata)
   expect_true(is.na(res$x[1]))
-  expect_is(res$x, "factor")
+  expect_s3_class(res$x, "factor")
 })
 
-test_that("bind_rows handles complex. #933", {
+test_that("bind_rows() handles complex. #933", {
   df1 <- data.frame(r = c(1 + 1i, 2 - 1i))
   df2 <- data.frame(r = c(1 - 1i, 2 + 1i))
   df3 <- bind_rows(df1, df2)
@@ -262,7 +262,7 @@ test_that("bind_rows handles complex. #933", {
   expect_equal(df3$r, c(df1$r, df2$r))
 })
 
-test_that("bind_rows is careful about column names encoding #1265", {
+test_that("bind_rows() is careful about column names encoding #1265", {
   one <- data.frame(foo = 1:3, bar = 1:3)
   names(one) <- c("f\u00fc", "bar")
   two <- data.frame(foo = 1:3, bar = 1:3)
@@ -273,7 +273,7 @@ test_that("bind_rows is careful about column names encoding #1265", {
   expect_equal(ncol(res), 2L)
 })
 
-test_that("bind_rows handles POSIXct (#1125)", {
+test_that("bind_rows() handles POSIXct (#1125)", {
   df1 <- data.frame(date = as.POSIXct(NA))
   df2 <- data.frame(date = as.POSIXct("2015-05-05"))
   res <- bind_rows(df1, df2)
@@ -281,29 +281,29 @@ test_that("bind_rows handles POSIXct (#1125)", {
   expect_true(is.na(res$date[1]))
 })
 
-test_that("bind_rows respects ordered factors (#1112)", {
+test_that("bind_rows() respects ordered factors (#1112)", {
   l <- c("a", "b", "c", "d")
   id <- factor(c("a", "c", "d"), levels = l, ordered = TRUE)
   df <- data.frame(id = rep(id, 2), val = rnorm(6))
   res <- bind_rows(df, df)
-  expect_is(res$id, "ordered")
+  expect_s3_class(res$id, "ordered")
   expect_equal(levels(df$id), levels(res$id))
 
   res <- group_by(df, id) %>% filter(complete.cases(across()))
-  expect_is(res$id, "ordered")
+  expect_s3_class(res$id, "ordered")
   expect_equal(levels(df$id), levels(res$id))
 })
 
-test_that("bind_rows keeps ordered factors (#948)", {
+test_that("bind_rows() keeps ordered factors (#948)", {
   y <- bind_rows(
     data.frame(x = factor(c(1, 2, 3), ordered = TRUE)),
     data.frame(x = factor(c(1, 2, 3), ordered = TRUE))
   )
-  expect_is(y$x, "ordered")
+  expect_s3_class(y$x, "ordered")
   expect_equal(levels(y$x), as.character(1:3))
 })
 
-test_that("bind handles POSIXct of different tz ", {
+test_that("bind_rows() handles POSIXct of different tz ", {
   date1 <- structure(-1735660800, tzone = "America/Chicago", class = c("POSIXct", "POSIXt"))
   date2 <- structure(-1735660800, tzone = "UTC", class = c("POSIXct", "POSIXt"))
   date3 <- structure(-1735660800, class = c("POSIXct", "POSIXt"))
@@ -354,7 +354,7 @@ test_that("string vectors are filled with NA not blanks before collection (#595)
   expect_true(all(is.na(res$char_col[1:10])))
 })
 
-test_that("bind_rows handles POSIXct stored as integer (#1402)", {
+test_that("bind_rows() handles POSIXct stored as integer (#1402)", {
   now <- Sys.time()
 
   df1 <- data.frame(time = now)
@@ -368,7 +368,7 @@ test_that("bind_rows handles POSIXct stored as integer (#1402)", {
   expect_true(all(res$time == c(df1$time, df2$time)))
 })
 
-test_that("bind_cols accepts NULL (#1148)", {
+test_that("bind_cols() accepts NULL (#1148)", {
   df1 <- tibble(a = 1:10, b = 1:10)
   df2 <- tibble(c = 1:10, d = 1:10)
 
@@ -382,14 +382,14 @@ test_that("bind_cols accepts NULL (#1148)", {
   expect_identical(res1, res4)
 })
 
-test_that("bind_rows handles 0-length named list (#1515)", {
+test_that("bind_rows() handles 0-length named list (#1515)", {
   res <- bind_rows(list(a = 1)[-1])
   expect_equal(nrow(res), 0L)
-  expect_is(res, "data.frame")
+  expect_s3_class(res, "data.frame")
   expect_equal(ncol(res), 0L)
 })
 
-test_that("bind_rows infers classes from first result (#1692)", {
+test_that("bind_rows() infers classes from first result (#1692)", {
   d1 <- data.frame(a = 1:10, b = rep(1:2, each = 5))
   d2 <- as_tibble(d1)
   d3 <- group_by(d1, b)
@@ -404,15 +404,17 @@ test_that("bind_rows infers classes from first result (#1692)", {
   expect_equal(class(bind_rows(d4, d1)), c("rowwise_df", "tbl_df", "tbl", "data.frame"))
 })
 
-test_that("bind_cols infers classes from first result (#1692)", {
+test_that("bind_cols() infers classes from first result (#1692)", {
   d1 <- data.frame(a = 1:10, b = rep(1:2, each = 5))
   d2 <- tibble(c = 1:10, d = rep(1:2, each = 5))
   d3 <- group_by(d2, d)
   d4 <- rowwise(d2)
   d5 <- list(c = 1:10, d = rep(1:2, each = 5))
 
-  expect_equal(class(bind_cols(d1, d1)), "data.frame")
-  expect_equal(class(bind_cols(d2, d1)), c("tbl_df", "tbl", "data.frame"))
+  suppressMessages({
+    expect_equal(class(bind_cols(d1, d1)), "data.frame")
+    expect_equal(class(bind_cols(d2, d1)), c("tbl_df", "tbl", "data.frame"))
+  })
   res3 <- bind_cols(d3, d1)
   expect_equal(class(res3), c("grouped_df", "tbl_df", "tbl", "data.frame"))
   expect_equal(map_int(group_rows(res3), length), c(5, 5))
@@ -420,7 +422,7 @@ test_that("bind_cols infers classes from first result (#1692)", {
   expect_equal(class(bind_cols(d5, d1)), "data.frame")
 })
 
-test_that("bind_rows accepts data frame columns (#2015)", {
+test_that("bind_rows() accepts data frame columns (#2015)", {
   df <- list(
     x = 1:10,
     y = data.frame(a = 1:10, y = 1:10)
@@ -429,11 +431,11 @@ test_that("bind_rows accepts data frame columns (#2015)", {
   attr(df, "row.names") <- .set_row_names(10)
 
   res <- dplyr::bind_rows(df, df)
-  expect_is(df$y, "data.frame")
+  expect_s3_class(df$y, "data.frame")
   expect_equal(names(df$y), c("a", "y"))
 })
 
-test_that("bind_rows accepts difftime objects", {
+test_that("bind_rows() accepts difftime objects", {
   df1 <- data.frame(x = as.difftime(1, units = "hours"))
   df2 <- data.frame(x = as.difftime(1, units = "mins"))
   res <- bind_rows(df1, df2)
@@ -448,7 +450,11 @@ test_that("bind_rows() handles rowwise vectors", {
   expect_identical(tbl, tibble(a = c("foo", "A"), b = c("bar", "B")))
 
   id_tbl <- bind_rows(a = c(a = 1, b = 2), b = c(a = 3, b = 4), .id = "id")
-  expect_equivalent(id_tbl, tibble(id = c("a", "b"), a = c(1, 3), b = c(2, 4)))
+  expect_equal(
+    id_tbl,
+    tibble(id = c("a", "b"), a = c(1, 3), b = c(2, 4)),
+    ignore_attr = TRUE
+  )
 })
 
 test_that("bind_rows() accepts lists of dataframe-like lists as first argument", {
@@ -457,17 +463,17 @@ test_that("bind_rows() accepts lists of dataframe-like lists as first argument",
   expect_equal(bind_rows(list(ll, ll)), tibble(a = c(1, 1), b = c(2, 2)))
 })
 
-test_that("bind_rows can handle lists (#1104)", {
+test_that("bind_rows() can handle lists (#1104)", {
   my_list <- list(list(x = 1, y = "a"), list(x = 2, y = "b"))
   res <- bind_rows(my_list)
   expect_equal(nrow(res), 2L)
-  expect_is(res$x, "numeric")
-  expect_is(res$y, "character")
+  expect_type(res$x, "double")
+  expect_type(res$y, "character")
 
   res <- bind_rows(list(x = 1, y = "a"), list(x = 2, y = "b"))
   expect_equal(nrow(res), 2L)
-  expect_is(res$x, "numeric")
-  expect_is(res$y, "character")
+  expect_type(res$x, "double")
+  expect_type(res$y, "character")
 })
 
 test_that("columns that are OBJECT but have NULL class are handled gracefully (#3349)", {
@@ -489,10 +495,9 @@ test_that("ignores NULL values", {
 })
 
 test_that("bind_cols() handles unnamed list with name repair (#3402)", {
-  expect_identical(
-    bind_cols(list(1, 2)),
-    bind_cols(list(...1 = 1, ...2 = 2))
-  )
+  expect_snapshot(df <- bind_cols(list(1, 2)))
+
+  expect_identical(df, bind_cols(list(...1 = 1, ...2 = 2)))
 })
 
 test_that("bind_cols() doesn't squash record types", {
@@ -516,8 +521,8 @@ test_that("bind_rows() only flattens list subclasses with explicit inheritance (
 })
 
 test_that("bind_rows() handles named list", {
-  expect_equivalent(bind_rows(map(mtcars, mean)), summarise_all(mtcars, mean))
-  expect_equivalent(bind_rows(!!!map(mtcars, mean)), summarise_all(mtcars, mean))
+  expect_equal(bind_rows(map(mtcars, mean)), summarise_all(mtcars, mean), ignore_attr = TRUE)
+  expect_equal(bind_rows(!!!map(mtcars, mean)), summarise_all(mtcars, mean), ignore_attr = TRUE)
 })
 
 test_that("bind_rows() handles named S3 objects (#4931)", {
@@ -542,46 +547,118 @@ test_that("bind_rows() correctly restores (#2457)", {
   df <- bind_rows(
     tibble(x = vctrs::list_of(1))
   )
-  expect_is(df$x, "vctrs_list_of")
+  expect_s3_class(df$x, "vctrs_list_of")
+})
+
+test_that("bind_rows() validates lists (#5417)", {
+  out <- bind_rows(list(x = 1), list(x = 1, y = 1:2))
+  expect_identical(out, tibble(x = c(1, 1, 1), y = c(NA, 1:2)))
+
+  x <- vctrs::list_of(a = data.frame(x = 1), b = data.frame(y = 2:3))
+  out <- bind_rows(x)
+  exp <- tibble(
+    a = vctrs::data_frame(x = c(1, 1), y = int(NA, NA)),
+    b = vctrs::data_frame(x = dbl(NA, NA), y = 2:3)
+  )
+  expect_identical(out, exp)
+})
+
+test_that("bind_rows() handles missing, null, and empty elements (#5429)", {
+  x <- list(a = "A", b = 1)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_identical(
+    bind_rows(l),
+    tibble(a = c("A", "B"), b = c(1, 2))
+  )
+
+  x <- list(a = NA, b = NA)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_identical(
+    bind_rows(l),
+    tibble(a = c(NA, "B"), b = c(NA, 2))
+  )
+
+  x <- list(a = NULL, b = NULL)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_identical(
+    bind_rows(l),
+    tibble(a = "B", b = 2)
+  )
+
+  x <- list(a = NULL, b = 1)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_identical(
+    bind_rows(l),
+    tibble(b = c(1, 2), a = c(NA, "B"))
+  )
+
+  x <- list(a = character(0), b = 1)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_identical(
+    bind_rows(l),
+    tibble(a = "B", b = 2)
+  )
+
+  x <- list(a = character(0), b = 1:2)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_error(
+    bind_rows(l),
+    class = "vctrs_error_incompatible_size"
+  )
+
+  x <- list(a = letters[1:3], b = 1:2)
+  y <- list(a = "B", b = 2)
+  l <- list(x, y)
+  expect_error(
+    bind_rows(l),
+    class = "vctrs_error_incompatible_size"
+  )
 })
 
 
 # Errors ------------------------------------------------------------------
 
 test_that("*_bind() give meaningful errors", {
-  verify_output(test_path("test-bind-errors.txt"), {
-    "# invalid .id"
+  expect_snapshot({
+    # invalid .id
     df1 <- tibble(x = 1:3)
     df2 <- tibble(x = 4:6)
-    bind_rows(df1, df2, .id = 5)
+    (expect_error(bind_rows(df1, df2, .id = 5)))
 
-    "# invalid type"
+    # invalid type"
     ll <- list(1:5, env(a = 1))
-    bind_rows(ll)
+    (expect_error(bind_rows(ll)))
 
     ll <- list(tibble(a = 1:5), env(a = 1))
-    bind_rows(ll)
+    (expect_error(bind_rows(ll)))
 
     df1 <- tibble(a = factor("a"))
     df2 <- tibble(a = 1L)
     df3 <- tibble(a = 1)
-    bind_rows(df1, df2)
-    bind_rows(df1, df3)
+    (expect_error(bind_rows(df1, df2)))
+    (expect_error(bind_rows(df1, df3)))
 
     df1 <- tibble(b = c(1, 2))
     df2 <- tibble(b = c(1L, 2L))
     df3 <- tibble(b = factor(c("A", "B")))
     df4 <- tibble(b = c("C", "D"))
-    bind_rows(df1, df3)
-    bind_rows(df1, df4)
-    bind_rows(df2, df3)
-    bind_rows(df2, df4)
+    (expect_error(bind_rows(df1, df3)))
+    (expect_error(bind_rows(df1, df4)))
+    (expect_error(bind_rows(df2, df3)))
+    (expect_error(bind_rows(df2, df4)))
 
     "# unnamed vectors"
-    bind_rows(1:2)
+    (expect_error(bind_rows(1:2)))
 
     "# incompatible size"
-    bind_cols(a = 1:2, mtcars)
-    bind_cols(mtcars, a = 1:3)
+    (expect_error(bind_cols(a = 1:2, mtcars)))
+    (expect_error(bind_cols(mtcars, a = 1:3)))
   })
+
 })
