@@ -1,14 +1,7 @@
-context("colwise select")
-
 df <- tibble(x = 0L, y = 0.5, z = 1)
 
 test_that("can select/rename all variables", {
   expect_identical(select_all(df), df)
-  expect_error(
-    rename_all(df),
-    "`.funs` must specify a renaming function",
-    fixed = TRUE
-  )
 
   expect_identical(select_all(df, toupper), set_names(df, c("X", "Y", "Z")))
   expect_identical(select_all(df, toupper), rename_all(df, toupper))
@@ -16,58 +9,24 @@ test_that("can select/rename all variables", {
 
 test_that("can select/rename with predicate", {
   expect_identical(select_if(df, is_integerish), select(df, x, z))
-  expect_error(
-    rename_if(df, is_integerish),
-    "`.funs` must specify a renaming function",
-    fixed = TRUE
-  )
-
   expect_identical(select_if(df, is_integerish, toupper), set_names(df[c("x", "z")], c("X", "Z")))
   expect_identical(rename_if(df, is_integerish, toupper), set_names(df, c("X", "y", "Z")))
 })
 
-test_that("can supply funs()", {
-  scoped_lifecycle_silence()
-  expect_identical(select_if(df, funs(is_integerish(.)), funs(toupper(.))), set_names(df[c("x", "z")], c("X", "Z")))
-  expect_identical(rename_if(df, funs(is_integerish(.)), funs(toupper(.))), set_names(df, c("X", "y", "Z")))
-
-  expect_identical(select_if(df, list(~is_integerish(.)), list(~toupper(.))), set_names(df[c("x", "z")], c("X", "Z")))
-  expect_identical(rename_if(df, list(~is_integerish(.)), list(~toupper(.))), set_names(df, c("X", "y", "Z")))
-})
-
-test_that("fails when more than one renaming function is supplied", {
-  scoped_lifecycle_silence()
-  expect_error(
-    select_all(df, funs(tolower, toupper)),
-    "`.funs` must contain one renaming function, not 2",
-    fixed = TRUE
-  )
-  expect_error(
-    rename_all(df, funs(tolower, toupper)),
-    "`.funs` must contain one renaming function, not 2",
-    fixed = TRUE
+test_that("can take list, but only containing single function", {
+  expect_identical(
+    select_if(df, list(~ is_integerish(.)), list(~ toupper(.))),
+    set_names(df[c("x", "z")], c("X", "Z"))
   )
 
-  expect_error(
-    select_all(df, list(tolower, toupper)),
-    "`.funs` must contain one renaming function, not 2",
-    fixed = TRUE
-  )
-  expect_error(
-    rename_all(df, list(tolower, toupper)),
-    "`.funs` must contain one renaming function, not 2",
-    fixed = TRUE
+  expect_identical(
+    rename_if(df, list(~ is_integerish(.)), list(~ toupper(.))),
+    set_names(df, c("X", "y", "Z"))
   )
 })
 
 test_that("can select/rename with vars()", {
   expect_identical(select_at(df, vars(x:y)), df[-3])
-  expect_error(
-    rename_at(df, vars(x:y)),
-    "`.funs` must specify a renaming function",
-    fixed = TRUE
-  )
-
   expect_identical(select_at(df, vars(x:y), toupper), set_names(df[-3], c("X", "Y")))
   expect_identical(rename_at(df, vars(x:y), toupper), set_names(df, c("X", "Y", "z")))
 })
@@ -91,8 +50,9 @@ test_that("select variants can use grouping variables (#3351, #3480)", {
 })
 
 test_that("select_if keeps grouping cols", {
-  expect_silent(df <- iris %>% group_by(Species) %>% select_if(is.numeric))
-  expect_equal(df, tbl_df(iris[c(5, 1:4)]))
+  by_species <- iris %>% group_by(Species)
+  expect_silent(df <- by_species %>% select_if(is.numeric))
+  expect_equal(df, by_species[c(5, 1:4)])
 })
 
 test_that("select_if() handles non-syntactic colnames", {
@@ -149,13 +109,13 @@ test_that("rename variants can rename a grouping variable (#3351)", {
 })
 
 test_that("select_all does not change the order of columns (#3351)", {
-  tbl <- group_by(tibble(x = 1:4, y = 1:4), y)
+  tbl <- group_by(tibble(x = 1:4, y = 4:1), y)
   expect_identical(select_all(tbl), tbl)
 
-  tbl <- group_by(tibble(x = 1:4, y = 1:4), x)
+  tbl <- group_by(tibble(x = 1:4, y = 4:1), x)
   expect_identical(select_all(tbl), tbl)
 
-  tbl <- group_by(tibble(x = 1:4, y = 1:4, z = 1:4), y)
+  tbl <- group_by(tibble(x = 1:4, y = 4:1, z = 1:4), y)
   expect_identical(select_all(tbl), tbl)
 })
 
@@ -190,3 +150,58 @@ test_that("rename_at() handles empty selection (#4324)", {
     mtcars
   )
 })
+
+test_that("rename_all/at() call the function with simple character vector (#4459)", {
+  fun <- function(x) case_when(x == 'mpg' ~ 'fuel_efficiency', TRUE ~ x)
+  out <- rename_all(mtcars,fun)
+  expect_equal(names(out)[1L], 'fuel_efficiency')
+
+  out <- rename_at(mtcars, vars(everything()), fun)
+  expect_equal(names(out)[1L], 'fuel_efficiency')
+})
+
+test_that("select_if() discards the column when predicate gives NA (#4486)", {
+  out <- tibble(mycol=c("","",NA)) %>% select_if(~!all(.==""))
+  expect_identical(
+    out,
+    tibble::new_tibble(list(), nrow = 3L)
+  )
+})
+
+# Errors ------------------------------------------------------------------
+
+test_that("colwise select() / rename() give meaningful errors", {
+  expect_snapshot({
+    df <- tibble(x = 0L, y = 0.5, z = 1)
+
+    # colwise rename()
+    (expect_error(
+      df %>% rename_all()
+    ))
+    (expect_error(
+      df %>% rename_if(is_integerish)
+    ))
+    (expect_error(
+      df %>% rename_at(vars(x:y))
+    ))
+    (expect_error(
+      df %>% rename_all(list(tolower, toupper))
+    ))
+
+    # colwise select()
+    (expect_error(
+      df %>% select_all(list(tolower, toupper))
+    ))
+    (expect_error(
+      df %>% select_if(function(.x) 1)
+    ))
+    (expect_error(
+      df %>% select_if(function(.x) c(TRUE, TRUE))
+    ))
+
+    (expect_error(
+      data.frame() %>% select_all(.funs = 42)
+    ))
+  })
+
+})
