@@ -1,5 +1,3 @@
-context("Filter")
-
 test_that("filter handles passing ...", {
   df <- data.frame(x = 1:4)
 
@@ -51,7 +49,7 @@ test_that("filter handles simple symbols", {
 })
 
 test_that("filter handlers scalar results", {
-  expect_equivalent(filter(mtcars, min(mpg) > 0), mtcars)
+  expect_equal(filter(mtcars, min(mpg) > 0), mtcars, ignore_attr = TRUE)
   expect_equal(filter(group_by(mtcars, cyl), min(mpg) > 0), group_by(mtcars, cyl))
 })
 
@@ -251,8 +249,7 @@ test_that("filter handles S4 objects (#1366)", {
 
   df <- data.frame(x = Numbers(1:10, foo = 10))
   res <- filter(df, x > 3)
-  expect_true(isS4(res$x))
-  expect_is(res$x, "Numbers")
+  expect_s4_class(res$x, "Numbers")
   expect_equal(res$x@foo, 10)
 })
 
@@ -379,65 +376,113 @@ test_that("filter() allows named constants that resolve to logical vectors (#461
   )
 })
 
+test_that("filter() allowing matrices with 1 column", {
+  out <- expect_warning(
+    filter(data.frame(x = 1:2), matrix(c(TRUE, FALSE), nrow = 2)), NA
+  )
+  expect_identical(out, data.frame(x = 1L))
+})
+
 test_that("filter() gives useful error messages", {
-  verify_output(test_path("test-filter-errors.txt"), {
-    "# wrong type"
-    iris %>%
-      group_by(Species) %>%
-      filter(1:n())
-    iris %>%
-      filter(1:n())
+  expect_snapshot({
+    # wrong type
+    (expect_error(
+      iris %>%
+        group_by(Species) %>%
+        filter(1:n())
+    ))
+    (expect_error(
+      iris %>%
+        filter(1:n())
+    ))
 
-    "# wrong size"
-    iris %>%
-      group_by(Species) %>%
-      filter(c(TRUE, FALSE))
-    iris %>%
-      rowwise(Species) %>%
-      filter(c(TRUE, FALSE))
-    iris %>%
-      filter(c(TRUE, FALSE))
+    # matrix with > 1 columns
+    (expect_error(
+      filter(data.frame(x = 1:2), matrix(c(TRUE, FALSE, TRUE, FALSE), nrow = 2))
+    ))
 
-    "# wrong size in column"
-    iris %>%
-      group_by(Species) %>%
-      filter(data.frame(c(TRUE, FALSE)))
-    iris %>%
-      rowwise() %>%
-      filter(data.frame(c(TRUE, FALSE)))
-    iris %>%
-      filter(data.frame(c(TRUE, FALSE)))
-    tibble(x = 1) %>%
-      filter(c(TRUE, TRUE))
+    # wrong size
+    (expect_error(
+      iris %>%
+        group_by(Species) %>%
+        filter(c(TRUE, FALSE))
+    ))
+    (expect_error(
+                    iris %>%
+                      rowwise(Species) %>%
+                      filter(c(TRUE, FALSE))
+    ))
+    (expect_error(
+                    iris %>%
+                      filter(c(TRUE, FALSE))
+    ))
 
-    "# wrong type in column"
-    iris %>%
-      group_by(Species) %>%
-      filter(data.frame(Sepal.Length > 3, 1:n()))
-    iris %>%
-      filter(data.frame(Sepal.Length > 3, 1:n()))
+    # wrong size in column
+    (expect_error(
+                    iris %>%
+                      group_by(Species) %>%
+                      filter(data.frame(c(TRUE, FALSE)))
+    ))
+    (expect_error(
+                    iris %>%
+                      rowwise() %>%
+                      filter(data.frame(c(TRUE, FALSE)))
+    ))
+    (expect_error(
+                    iris %>%
+                      filter(data.frame(c(TRUE, FALSE)))
+    ))
+    (expect_error(
+                    tibble(x = 1) %>%
+                      filter(c(TRUE, TRUE))
+    ))
 
-    "# evaluation error"
-    mtcars %>%
-      filter(`_x`)
-    mtcars %>%
-      group_by(cyl) %>%
-      filter(`_x`)
+    # wrong type in column
+    (expect_error(
+                    iris %>%
+                      group_by(Species) %>%
+                      filter(data.frame(Sepal.Length > 3, 1:n()))
+    ))
+    (expect_error(
+                    iris %>%
+                      filter(data.frame(Sepal.Length > 3, 1:n()))
+    ))
 
-    "# named inputs"
-    filter(mtcars, x = 1)
-    filter(mtcars, y > 2, z = 3)
-    filter(mtcars, TRUE, x = 1)
+    # evaluation error
+    (expect_error(
+      mtcars %>% filter(`_x`)
+    ))
+    (expect_error(
+                    mtcars %>%
+                      group_by(cyl) %>%
+                      filter(`_x`)
+    ))
 
-    "# ts "
-    filter(ts(1:10))
+    # named inputs
+    (expect_error(
+      filter(mtcars, x = 1)
+    ))
+    (expect_error(
+      filter(mtcars, y > 2, z = 3)
+    ))
+    (expect_error(
+      filter(mtcars, TRUE, x = 1)
+    ))
 
-    "# Error that contains {"
-    tibble() %>% filter(stop("{"))
+    # ts
+    (expect_error(
+      filter(ts(1:10))
+    ))
 
-    "# across() in filter() does not warn yet"
+    # Error that contains {
+    (expect_error(
+      tibble() %>% filter(stop("{"))
+    ))
+
+    # across() in filter() does not warn yet
     data.frame(x = 1, y = 1) %>%
       filter(across(everything(), ~ .x > 0))
+
     data.frame(x = 1, y = 1) %>%
       filter(data.frame(x > 0, y > 0))
   })
