@@ -1,8 +1,6 @@
-context("Group by")
-
 df <- data.frame(x = rep(1:3, each = 10), y = rep(1:6, each = 5))
 
-test_that("group_by with .add = TRUE adds groups", {
+test_that("group_by() with .add = TRUE adds groups", {
   add_groups1 <- function(tbl) group_by(tbl, x, y, .add = TRUE)
   add_groups2 <- function(tbl) group_by(group_by(tbl, x, .add = TRUE), y, .add = TRUE)
 
@@ -10,8 +8,39 @@ test_that("group_by with .add = TRUE adds groups", {
   expect_equal(group_vars(add_groups2(df)), c("x", "y"))
 })
 
+test_that("group_by(<grouped df>, <computation>) computes the expressions on the ungrouped data frame (#5938)", {
+  df <- data.frame(
+    x = 1:4,
+    g = rep(1:2, each = 2)
+  )
+
+  count <- 0
+  out <- df %>% group_by(g) %>% group_by(big = { count <<- count + 1; x > mean(x) })
+  expect_equal(out$big, c(FALSE, FALSE, TRUE, TRUE))
+  expect_equal(count, 1L)
+  expect_equal(group_vars(out), c("big"))
+
+  count <- 0
+  out <- df %>% group_by(g) %>% group_by(big = { count <<- count + 1; x > mean(x) }, .add = TRUE)
+  expect_equal(out$big, c(FALSE, FALSE, TRUE, TRUE))
+  expect_equal(count, 1L)
+  expect_equal(group_vars(out), c("g", "big"))
+
+  count <- 0
+  out <- df %>% group_by(g) %>% mutate(big = { count <<- count + 1; x > mean(x)}) %>% group_by(big)
+  expect_equal(out$big, c(FALSE, TRUE, FALSE, TRUE))
+  expect_equal(count, 2L)
+  expect_equal(group_vars(out), c("big"))
+
+  count <- 0
+  out <- df %>% group_by(g) %>% mutate(big = { count <<- count + 1; x > mean(x)}) %>% group_by(big, .add = TRUE)
+  expect_equal(out$big, c(FALSE, TRUE, FALSE, TRUE))
+  expect_equal(count, 2L)
+  expect_equal(group_vars(out), c("g", "big"))
+})
+
 test_that("add = TRUE is deprecated", {
-  rlang::scoped_options(lifecycle_verbosity = "warning")
+  rlang::local_options(lifecycle_verbosity = "warning")
 
   df <- tibble(x = 1, y = 2)
 
@@ -43,7 +72,7 @@ test_that("grouping by constant adds column (#410)", {
 })
 
 test_that(".dots is soft deprecated", {
-  rlang::scoped_options(lifecycle_verbosity = "warning")
+  rlang::local_options(lifecycle_verbosity = "warning")
 
   df <- tibble(x = 1, y = 1)
   expect_warning(gf <- group_by(df, .dots = "x"), "deprecated")
@@ -72,10 +101,10 @@ test_that("local group_by preserves variable types", {
   }
 })
 
-test_that("mutate does not loose variables (#144)", {
+test_that("mutate does not lose variables (#144)", {
   df <- tibble(a = rep(1:4, 2), b = rep(1:4, each = 2), x = runif(8))
   by_ab <- group_by(df, a, b)
-  by_a <- summarise(by_ab, x = sum(x))
+  by_a <- summarise(by_ab, x = sum(x), .groups = "drop_last")
   by_a_quartile <- group_by(by_a, quartile = ntile(x, 4))
 
   expect_equal(names(by_a_quartile), c("a", "b", "x", "quartile"))
@@ -118,7 +147,9 @@ test_that("group_by() handles list as grouping variables", {
 })
 
 test_that("select(group_by(.)) implicitely adds grouping variables (#170)", {
-  res <- mtcars %>% group_by(vs) %>% select(mpg)
+  expect_snapshot(
+    res <- mtcars %>% group_by(vs) %>% select(mpg)
+  )
   expect_equal(names(res), c("vs", "mpg"))
 })
 
@@ -164,7 +195,9 @@ test_that("group_by works with zero-row data frames (#486)", {
   expect_equal(group_vars(x), "g")
   expect_equal(group_size(x), integer(0))
 
-  x <- select(dfg, a) # Only select 'a' column; should result in 'g' and 'a'
+  expect_snapshot(
+    x <- select(dfg, a) # Only select 'a' column; should result in 'g' and 'a'
+  )
   expect_equal(dim(x), c(0, 2))
   expect_equal(group_vars(x), "g")
   expect_equal(group_size(x), integer(0))
@@ -185,7 +218,7 @@ test_that("[ on grouped_df drops grouping if subset doesn't include grouping var
   no_cyl <- by_cyl %>% `[`(c(1, 3))
 
   expect_equal(group_vars(no_cyl), character())
-  expect_is(no_cyl, "tbl_df")
+  expect_s3_class(no_cyl, "tbl_df")
 })
 
 test_that("group_by works after arrange (#959)", {
@@ -242,7 +275,7 @@ test_that("group_by handles raw columns (#1803)", {
 
 test_that("rowwise handles raw columns (#1803)", {
   df <- tibble(a = 1:3, b = as.raw(1:3))
-  expect_is(rowwise(df), "rowwise_df")
+  expect_s3_class(rowwise(df), "rowwise_df")
 })
 
 test_that("group_by() names pronouns correctly (#2686)", {
@@ -407,7 +440,7 @@ test_that("summarise maintains the .drop attribute (#4061)", {
     group_by(f1, f2, .drop = TRUE)
   expect_equal(n_groups(res), 1L)
 
-  res2 <- summarise(res, x = sum(x))
+  res2 <- summarise(res, x = sum(x), .groups = "drop_last")
   expect_equal(n_groups(res2), 1L)
   expect_true(group_by_drop_default(res2))
 })
@@ -423,14 +456,14 @@ test_that("joins maintain the .drop attribute (#4061)", {
     y = 1
   ), f1, .drop = TRUE)
 
-  res <- left_join(df1, df2)
+  res <- left_join(df1, df2, by = "f1")
   expect_equal(n_groups(res), 2L)
 
   df2 <- group_by(tibble(
     f1 = factor(c("a", "c"), levels = c("a", "b", "c")),
     y = 1:2
   ), f1, .drop = TRUE)
-  res <- full_join(df1, df2)
+  res <- full_join(df1, df2, by = "f1")
   expect_equal(n_groups(res), 3L)
 })
 
@@ -522,18 +555,54 @@ test_that("implicit mutate() operates on ungrouped data (#5598)", {
   expect_equal(vars, c("y", "z"))
 })
 
+test_that("grouped_df() does not break row.names (#5745)", {
+  groups <- compute_groups(data.frame(x = 1:10), "x")
+  expect_equal(.row_names_info(groups, type = 0), c(NA, -10L))
+})
+
+test_that("group_by() keeps attributes unrelated to the grouping (#5760)", {
+  d <- data.frame(x = 453, y = 642)
+  attr(d, "foo") <- "bar"
+
+  d2 <- group_by(d, x)
+  expect_equal(attr(d2, "foo"), "bar")
+
+  d3 <- group_by(d2, y, .add = TRUE)
+  expect_equal(attr(d2, "foo"), "bar")
+
+  d4 <- group_by(d2, y2 = y * 2, .add = TRUE)
+  expect_equal(attr(d2, "foo"), "bar")
+})
+
+test_that("group_by() works with quosures (tidyverse/lubridate#959)", {
+  ignore <- function(...) NA
+  f <- function(var) {
+    tibble(x = 1) %>% group_by(g = ignore({{ var }}))
+  }
+  g <- function(var) {
+    # This used to fail with the extra argument
+    tibble(x = 1) %>% group_by(g = ignore({{ var }}, 1))
+  }
+  expect_equal(f(), tibble(x = 1, g = NA) %>% group_by(g))
+  expect_equal(g(), tibble(x = 1, g = NA) %>% group_by(g))
+})
+
+test_that("group_by() propagates caller env", {
+  expect_caller_env(group_by(mtcars, sig_caller_env()))
+})
+
+
 # Errors ------------------------------------------------------------------
 
 test_that("group_by() and ungroup() give meaningful error messages", {
-  verify_output(test_path("test-group-by-errors.txt"), {
+  expect_snapshot({
     df <- tibble(x = 1, y = 2)
 
-    df %>% group_by(unknown)
+    (expect_error(df %>% group_by(unknown)))
+    (expect_error(df %>% ungroup(x)))
+    (expect_error(df %>% group_by(x, y) %>% ungroup(z)))
 
-    df %>% ungroup(x)
-    df %>% group_by(x, y) %>% ungroup(z)
-
-    df %>% group_by(z = a + 1)
+    (expect_error(df %>% group_by(z = a + 1)))
   })
-})
 
+})
